public class BinarySearchTree<K extends Comparable<K>, V> {
	private class Node {
		private K key;
		private V value;
		private Node left, right;

		private Node(K key, V data) {
			this.key = key;
			this.value = data;
		}
	}

	private Node root;

	public boolean contains(K key) {
		return containsHelper(key, root);
	}

	private boolean containsHelper(K key, Node localRoot) {
		if (localRoot == null) {
			return false;
		}
		int comparison = key.compareTo(localRoot.key);
		if (comparison == 0) {          // found in this node
			return true;          
		} else if (comparison < 0) {    // search left subtree
			return containsHelper(key, localRoot.left);
		} else {
			return containsHelper(key, localRoot.right);
		}
	}
	
	// assume low is less than high
	public int countRange(K low, K high) {
		return countRangeHelper(low, high, root);
	}
	
	private int countRangeHelper(K low, K high, Node localRoot) {
		if (localRoot == null) {
			return 0;
		}
		int count = 0;
		if (low.compareTo(localRoot.key) <= 0 && high.compareTo(localRoot.key) >= 0) {
			count++;               // current value is in range
		}
		if (low.compareTo(localRoot.key) < 0) {   // might be values in left subtree
			count += countRangeHelper(low, high, localRoot.left);
		}
		if (high.compareTo(localRoot.key) > 0) {  // might be values in right subtree
			count += countRangeHelper(low, high, localRoot.right);
		}
		return count;
	}

}