import java.util.NoSuchElementException;

public class Queue<T> {

	public T[] data;        // circular array
	
	private int size;       // number of elements being stored
							// Note that this is NOT the same as
	                        // the size of the array
	
	public int headIndex;   // position of first element
	
	private final int INITIAL_ARRAY_SIZE = 4;
	
	@SuppressWarnings("unchecked")
	public Queue() {
		data = (T[]) new Object[INITIAL_ARRAY_SIZE];
		size = 0;
		headIndex = 0;
	}
	
	public int getSize() {
		return size;
	}

	/* Adds an element to the circular array.
	 * 
	 * If the array becomes full, doubles the length 
	 * of the array (and moves the data to the front).
	 */
	@SuppressWarnings("unchecked")
	public void enqueue(T x) {
		if (size == data.length) {
			T[] b = (T[]) new Object[data.length * 2];
			for (int i = 0; i < size; i++) {
				b[i] = data[(i + headIndex) % data.length];
			}
			data = b;
			headIndex = 0;
		}
		data[(headIndex + size ) % data.length] = x;
		size++;
	}
	
	/* Removes an element from the queue and returns it.
	 * 
	 * If the queue is empty, throws a NoSuchElementException.
	 * 
	 * At the moment the very last element is removed,
	 * the headIndex is reset to 0.
	 */
	public T dequeue() {
		if (size == 0) {
			throw new NoSuchElementException();
		}
		T retValue = data[headIndex];
		data[headIndex] = null;
		headIndex++;
		if (headIndex == data.length ) {
			headIndex = 0;
		}
		size--;
		if (size == 0) {
			headIndex = 0;
		}
		return retValue;
	}
}
